@@ -1,5 +1,4 @@
-# @(#)$Id: CPANTesting.pm 1316 2013-05-14 17:48:52Z pjf $
-# Bob-Version: 1.7
+# @(#)Ident: CPANTesting.pm 2013-07-30 12:50 pjf ;
 
 package CPANTesting;
 
@@ -16,16 +15,18 @@ sub should_abort {
    is_testing() or return 0;
 
    $host eq q(xphvmfred) and return
-      "Terminated Stauner ${host} - cc06993e-a5e9-11e2-83b7-87183f85d660";
-
+      "ABORT: ${host} - cc06993e-a5e9-11e2-83b7-87183f85d660";
    return 0;
 }
 
 sub test_exceptions {
-   my $p = shift; is_testing() or return 0;
+   my $p = shift; my $perl_ver = $p->{requires}->{perl};
 
-   $p->{stop_tests}     and return 'CPAN Testing stopped in Build.PL';
-   $osname eq q(mirbsd) and return 'Mirbsd OS unsupported';
+   is_testing()         or  return 0;
+   $] < $perl_ver       and return "TESTS: Perl minimum ${perl_ver}";
+   $p->{stop_tests}     and return 'TESTS: CPAN Testing stopped in Build.PL';
+   $osname eq q(mirbsd) and return 'TESTS: Mirbsd OS unsupported';
+#  $host   eq q(broken) and return "tests: <CPAN Testing uuid>";
    return 0;
 }
 
