@@ -1,40 +1,39 @@
-# @(#)$Id: View.pm 1181 2012-04-17 19:06:07Z pjf $
+# @(#)$Id: View.pm 1320 2013-07-31 17:31:20Z pjf $
 
 package CatalystX::Usul::View;
 
 use strict;
-use warnings;
-use version; our $VERSION = qv( sprintf '0.7.%d', q$Rev: 1181 $ =~ /\d+/gmx );
-use parent qw(Catalyst::View CatalystX::Usul);
+use version; our $VERSION = qv( sprintf '0.8.%d', q$Rev: 1320 $ =~ /\d+/gmx );
 
+use CatalystX::Usul::Moose;
 use CatalystX::Usul::Constants;
-use CatalystX::Usul::Functions qw(exception is_arrayref is_hashref throw);
-use Encode;
-use HTML::FormWidgets;
-use Scalar::Util qw(blessed);
+use CatalystX::Usul::Functions   qw(exception is_arrayref is_hashref throw);
+use CatalystX::Usul::Constraints qw(Directory);
+use Scalar::Util                 qw(weaken);
 use Template;
 use TryCatch;
+use Encode;
 
-__PACKAGE__->config( form_sources    => [ qw(sdata hidden) ],
-                     js_object       => q(behaviour),
-                     serialize_attrs => {} );
+extends q(Catalyst::View);
+with    q(CatalystX::Usul::TraitFor::BuildingUsul);
 
-__PACKAGE__->mk_accessors( qw(content_types deserialize_attrs
-                              form_sources js_object serialize_attrs
-                              template_dir) );
+has 'deserialize_attrs' => is => 'ro',   isa => HashRef, default => sub { {} };
 
-sub COMPONENT {
-   my ($class, $app, @rest) = @_;
+has 'form_sources'      => is => 'ro',   isa => ArrayRef,
+   default              => sub { [ qw(sdata hidden) ] };
 
-   my $new  = $class->next::method( $app, @rest );
-   my $usul = CatalystX::Usul->new( $app, {} );
+has 'js_object'         => is => 'ro',   isa => NonEmptySimpleStr,
+   default              => q(behaviour);
 
-   for (grep { not defined $new->{ $_ } } keys %{ $usul }) {
-      $new->{ $_ } = $usul->{ $_ };
-   }
+has 'serialize_attrs'   => is => 'ro',   isa => HashRef, default => sub { {} };
 
-   return $new;
-}
+has 'template_dir'      => is => 'ro',   isa => Directory, coerce => TRUE;
+
+has 'usul'              => is => 'lazy', isa => BaseClass,
+   handles              => [ qw(debug encoding log) ];
+
+has 'widget_class'      => is => 'lazy', isa => LoadableClass, coerce => TRUE,
+   default              => sub { 'HTML::FormWidgets' };
 
 sub bad_request {
    my ($self, $c, $verb, $msg, $status) = @_; $status ||= 400;
@@ -46,16 +45,15 @@ sub bad_request {
 }
 
 sub deserialize {
-   my ($self, $s, $req, $process) = @_;
+   my ($self, $s, $req, $process) = @_; my $body; $s->{leader} = blessed $self;
 
-   my $body = $req->body; $s->{leader} = blessed $self;
-
-   if ($body) {
-      try        { return $process->( $self->deserialize_attrs || {}, $body ) }
-      catch ($e) { $self->log_error_message( (exception $e), $s ) }
+   if ($body = $req->body) {
+      try        { return $process->( $self->deserialize_attrs, $body ) }
+      catch ($e) { $self->log->error_message( $s, (exception $e) ) }
    }
    else {
-      $self->debug and $self->log_debug_message( 'Nothing to deserialize', $s );
+      $self->debug
+         and $self->log->debug_message( $s, 'Nothing to deserialize' );
    }
 
    return;
@@ -71,58 +69,30 @@ sub get_verb {
    return $verb;
 }
 
+sub loc { # Localize the key and substitute the placeholder args
+   my ($self, $opts, $key, @args) = @_; my $car = $args[ 0 ];
+
+   my $args = (is_hashref $car) ? { %{ $car } }
+            : { params => (is_arrayref $car) ? $car : [ @args ] };
+
+   $args->{domain_names} ||= [ DEFAULT_L10N_DOMAIN, $opts->{ns} ];
+   $args->{locale      } ||= $opts->{language};
+
+   return $self->usul->localize( $key, $args );
+}
+
 sub not_implemented {
-   my ($self, @rest) = @_; return $self->bad_request( @rest, 405 );
+   return shift->bad_request( @_, 404 );
 }
 
-sub process {
-   my ($self, $c) = @_; my $s = $c->stash;
+sub prepare_data {
+   my ($self, $c) = @_;
 
-   my $attrs = $self->serialize_attrs || {}; my $body;
-
-   my $type  = $attrs->{content_type} = $s->{content_type};
-
-   try        { $body  = $self->serialize( $attrs, $self->_prepare_data( $c )) }
-   catch ($e) { $body  = "Serializer ${type} failed\r\n***ERROR***\r\n";
-                $self->bad_request( $c, $body.(exception $e) ); return TRUE }
-
-   if (my $enc = $s->{encoding}) { # Encode the body of the page
-      $body = encode( $enc, $body ); $type .= q(; charset=).$enc;
-   }
-
-   $c->res->header( Vary => q(Content-Type) );
-   $c->res->content_type( $type );
-   $c->res->body( $body );
-   return TRUE;
-}
-
-# Private methods
-
-sub _build_widgets {
-   my ($self, $c, $args) = @_; my $s = $c->stash;
-
-   my $attrs = { %{ $args || {} } };
-   my @attrs = ( qw(assets content_type fields hidden
-                    literal_js optional_js pwidth width) );
-
-   $attrs->{ $_         } = $s->{ $_ } for (@attrs);
-   $attrs->{base        } = $c->req->base;
-   $attrs->{js_object   } = $self->js_object;
-   $attrs->{l10n        } = sub { $self->loc( $s, @_ ) };
-   $attrs->{root        } = $c->config->{root};
-   $attrs->{template_dir} = $self->template_dir;
-
-   HTML::FormWidgets->build( $attrs );
-   return;
-}
-
-sub _prepare_data {
-   my ($self, $c) = @_; my $s = $c->stash;
-
-   $s->{EOT} = $s->{is_xml} ? q( />) : q(>);
-
-   my $tt   = Template->new( {} ) or throw $Template::ERROR;
-   my $data = $self->_read_form_sources( $c ) || [];
+   my $data = $self->read_form_sources( $c ) || [];
+   my $s    = $c->stash; $s->{EOT} = $s->{is_xml} ? q( />) : q(>);
+   my $loc  = sub { return $self->loc( $s, @_ ) };
+   my $tt   = Template->new( { VARIABLES => { loc => $loc }, } )
+      or throw $Template::ERROR;
 
    for my $source (@{ $data }) {
       my $id = 0;
@@ -136,7 +106,7 @@ sub _prepare_data {
          }
          else { $item->{content} = __tt_process( $tt, $s, \$content ) }
 
-         defined $item->{id} or $item->{id} = $id; $id++;
+         $item->{id} //= $id; $id++;
       }
 
       $source->{count} = scalar @{ $source->{items} };
@@ -148,10 +118,36 @@ sub _prepare_data {
       push @{ $data->[ 0 ]->{items} || [] }, @{ $source->{items} || [] };
    }
 
+   $data->[ 0 ]->{count} = scalar @{ $data->[ 0 ]->{items} || [] };
+
    return $data->[ 0 ];
 }
 
-sub _read_form_sources {
+sub process {
+   my ($self, $c) = @_; my $s = $c->stash; my $type = $s->{content_type};
+
+   try {
+      my $attr = { %{ $self->serialize_attrs }, content_type => $type };
+      my $body = $self->serialize( $attr, $self->prepare_data( $c ) );
+
+      if (my $enc = $self->encoding) { # Encode the body of the page
+         $body = encode( $enc, $body ); $type .= "; charset=${enc}";
+      }
+
+      $c->res->body( $body );
+      $c->res->content_type( $type );
+      $c->res->header( Vary => q(Content-Type) );
+   }
+   catch ($e) {
+      my $body = "Serializer ${type} failed\r\n***ERROR***\r\n";
+
+      $self->bad_request( $c, $s->{verb}, $body.(exception $e) );
+   }
+
+   return TRUE;
+}
+
+sub read_form_sources {
    my ($self, $c) = @_; my $s = $c->stash;
 
    my $sources = $self->form_sources || []; my $data = [];
@@ -164,6 +160,26 @@ sub _read_form_sources {
    return $data;
 }
 
+# Private methods
+
+sub _build_widgets {
+   my ($self, $c, $args) = @_; my $s = $c->stash;
+
+   my $attr = { %{ $args || {} } };
+   my @attr = ( qw(assets content_type fields hidden
+                   language literal_js ns optional_js pwidth width) );
+
+   $attr->{ $_         } = $s->{ $_ } for (@attr);
+   $attr->{base        } = $c->req->base;
+   $attr->{js_object   } = $self->js_object;
+   $attr->{l10n        } = sub { $self->loc( @_ ) };
+   $attr->{root        } = $c->config->{root};
+   $attr->{template_dir} = $self->template_dir;
+
+   $self->widget_class->build( $attr );
+   return;
+}
+
 # Private subroutines
 
 sub __tt_process {
@@ -174,6 +190,8 @@ sub __tt_process {
    return $out;
 }
 
+__PACKAGE__->meta->make_immutable;
+
 1;
 
 __END__
@@ -186,25 +204,26 @@ CatalystX::Usul::View - Base class for views
 
 =head1 Version
 
-0.7.$Revision: 1181 $
+0.8.$Revision: 1320 $
 
 =head1 Synopsis
 
-   package CatalystX::Usul::View;
-   use parent qw(Catalyst::View CatalystX::Usul);
-
-   package CatalystX::Usul::View::HTML;
-   use parent qw(CatalystX::Usul::View);
-
    package YourApp::View::HTML;
-   use parent qw(CatalystX::Usul::View::HTML);
+
+   use CatalystX::Usul::Moose;
+
+   extends qw(CatalystX::Usul::View::HTML);
+
 
    package YourApp::View::JSON;
-   use parent qw(CatalystX::Usul::View::JSON);
+
+   use CatalystX::Usul::Moose;
+
+   extends qw(CatalystX::Usul::View::JSON);
 
 =head1 Description
 
-Provide common methods for subclasses
+Provide common methods for view component subclasses
 
 =head1 Subroutines/Methods
 
@@ -236,16 +255,38 @@ The verb is used by the action class to lookup the action to forward
 to. Called from the C<begin> method once the current view has been
 determined from the request content type
 
+=head2 loc
+
+   $localized_text = $self->loc( $c->stash, $key, @options );
+
+Localizes the message. Calls L<Class::Usul::L10N/localize>. Adds the
+constant C<DEFAULT_L10N_DOMAINS> to the list of domain files that are
+searched. Adds C<< $c->stash->language >> and C<< $c->stash->namespace >>
+(search domain) to the arguments passed to C<localize>
+
 =head2 not_implemented
 
 Sets the response body to the provided error message and the response
 status to 405
 
+=head2 prepare_data
+
+   $hash_ref = $self->prepare_data( $c );
+
+Called by L</process> this method is responsible for
+selecting those elements from the stash that are passed to
+the serializer method
+
 =head2 process
 
 Serializes the response using L<XML::Simple> and encodes the body using
 L<Encode> if required
 
+=head2 read_form_sources
+
+Returns an array ref widget references in the stash. Can be passed to
+L</_build_widgets> or its output can be sent directly to the serializer
+
 =head1 Private Methods
 
 =head2 _build_widgets
@@ -253,17 +294,6 @@ L<Encode> if required
 Calls C<build> in L<HTML::FormWidgets> which transforms the widgets
 definitions into fragments of HTML or XHTML as required
 
-=head2 _prepare_data
-
-Called by L</process> this method is responsible for
-selecting those elements from the stash that are passed to
-the serializer method
-
-=head2 _read_form_sources
-
-Returns an array ref widget references in the stash. Can be passed to
-L</_build_widgets> or its output can be sent directly to the serializer
-
 =head1 Diagnostics
 
 None
@@ -278,12 +308,20 @@ None
 
 =item L<Catalyst::View>
 
-=item L<CatalystX::Usul>
+=item L<Class::Usul>
+
+=item L<CatalystX::Usul::Moose>
 
 =item L<Encode>
 
+=item L<CatalystX::Usul::Constraints>
+
 =item L<HTML::FormWidgets>
 
+=item L<Template>
+
+=item L<TryCatch>
+
 =back
 
 =head1 Incompatibilities
@@ -302,7 +340,7 @@ Peter Flanigan, C<< <Support at RoxSoft.co.uk> >>
 
 =head1 License and Copyright
 
-Copyright (c) 2008 Peter Flanigan. All rights reserved
+Copyright (c) 2013 Peter Flanigan. All rights reserved
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself. See L<perlartistic>
